function [ff, beta, loglike, Tlambda, numNonzeros, iterations] = ...
      cvmt(lambda_input,nmain,beta_init,method,output)
% Tries to take full second-order step, but if this fails, scales the step
% along a line so that it stops at the first kink.
% method = 1 bgacv, method = 2 gacv
global lambda_global;
global fBasis;
global Y_global;
global nmain_global;
[sampsize nbeta] = size(fBasis);
lambda_global=lambda_input;
nmain_global = nmain;
lambda_globalv = [ones(nmain(1),1)*lambda_global(1);ones(nbeta-nmain(1),1)*lambda_global(2)];

%--------------------------------------------------------------------------
% Define some algorithmic parameters
%--------------------------------------------------------------------------
maxiter = 1000; 
tol = 1.e-6; 
alphaMax = 1.e12; 
%steplength_max = 20.0; 
%steplength_min = 1.e-8;
%scalGrad = 1.e-2;
iter = 0; %gpnorm = 1.0;
useNewton=1;                % set to 1 if want to have second-order scaling
                            % set to 0 if always use a gradient projection
two_metric_threshold = min(500,nbeta);
                            % ordinary GP is used above this threshhold
beta = beta_init;
Tlambda = Functionsmt(beta);
alpha = 1.e1;               % set initial value of the damping parameter
beta_step=zeros(nbeta,1);   % initialize beta_step;
loop_on=1;                  % set to zero to stop the loop

%--------------------------------------------------------------------------
% parameters to control randomized evaluation:
% sigma = fraction of random gradient components to evaluate for zero
%         components of beta. Set sigma=1 to get full gradient evaluation.
% evalFullGrad = flag indicating whether full gradient should be evaluated.
%                Turned on when gpnorm drops below tol.
%--------------------------------------------------------------------------
sigma = 0.01;
evalFullGrad = 0;

if output == 1
    nz=length(find(beta_init(1:nbeta)~=0));
    fprintf(' Initial point: nz=%d, f= %15.12g\n', nz, Tlambda);
end


%--------------------------------------------------------------------------
% Start the loop of the algorithm
%--------------------------------------------------------------------------
while (iter<maxiter) && (loop_on==1)
    iter = iter+1;
    
    % initialize beta_new to be the current beta (subsequently, some
    % of its entries will be modified by the step)
    beta_new=beta;      			    

    %----------------------------------------------------------------------
    % evaluate the gradient for (i) the current nonzero indices and (ii) a
    % fraction sigma of the other indices. 
    %----------------------------------------------------------------------
    if evalFullGrad
      beta_inx=1:nbeta;
    else
      beta_nz = find(beta);
      beta_rand = find(rand(nbeta,1)<sigma);
      beta_inx = union(beta_nz,beta_rand);
    end
    
    Grad = zeros(nbeta,1);
    Grad(beta_inx) = Gradient(beta, beta_inx);
    
    %----------------------------------------------------------------------
    % Evaluate convergence test quantity: subgradient of minimal norm
    %----------------------------------------------------------------------
    subGradMin = Grad - lambda_globalv.*(beta<0) + lambda_globalv.*(beta>0);
    km = find(beta==0 & Grad<=0);
    kp = find(beta==0 & Grad> 0);
    subGradMin(km) = min(0,Grad(km)+lambda_globalv(km));
    subGradMin(kp) = max(0,Grad(kp)-lambda_globalv(kp));
    subGradMin(nbeta) = Grad(nbeta);   % gradient of the last component 
                                       % remains the same
    gpnorm = norm(subGradMin);
    
    %----------------------------------------------------------------------
    % for interest, calculate the nonzero set for this beta
    %----------------------------------------------------------------------
    InonZero0 = find(beta);
    InonZero0 = union(InonZero0, nbeta);
    InonZero0_complement = setdiff(1:nbeta,InonZero0)';        
    numberNonZero0 = length(InonZero0);
    

    if gpnorm<tol
      if evalFullGrad==1
	break                 % break if convergence test is met
      else
	evalFullGrad=1;       % evaluate full gradient on next iteration
      end
    else
      evalFullGrad=0;         % turn off full grad evaluation (if necessary)
    end

    %----------------------------------------------------------------------
    % find the Barzilai-Borwein parameter, which approximates the curvature
    % of the Hessian along the latest step
    %----------------------------------------------------------------------
    if iter>1 && norm(beta_step)~=0
      alphaBB = (Grad-Grad_old)'*beta_step / (beta_step'*beta_step);
      % reset alpha if it appears to be too small - that is, make it at
      % least equal to curvature of the actual Hessian along the latest
      % step. This should keep the first-order step and reduced Newton step
      % in the same ballpark.
      if alphaBB > alpha
% 	fprintf(' resetting alpha to %10.4e to match curvature along latest step\n', alphaBB)
	alpha = alphaBB;
      end
    end


    %----------------------------------------------------------------------
    % Calculate the first-order step
    %----------------------------------------------------------------------
    betaGrad = Grad - alpha*beta;
    %k0 = find(betaGrad>=-lambda_global & betaGrad<=lambda_global);
    kn = find( betaGrad< -lambda_globalv);
    kp = find( betaGrad>  lambda_globalv);
    beta_new(betaGrad>=-lambda_globalv & betaGrad<=lambda_globalv) = 0.0;
    beta_new(kn) = beta(kn) - (Grad(kn)+lambda_globalv(kn))/alpha;
    beta_new(kp) = beta(kp) - (Grad(kp)-lambda_globalv(kp))/alpha;
    beta_new(nbeta) = beta(nbeta) - Grad(nbeta)/alpha;
                                       % last component is a special case
    beta_step_first = beta_new-beta;

    %----------------------------------------------------------------------
    % evaluate the function at the first-order point. The Newton step, if
    % tried, has to do better than Tlambda_first in order to be accepted.
    %----------------------------------------------------------------------
    Tlambda_first = Functionsmt(beta+beta_step_first);
    InonZero=find(beta_new);
    InonZero=[InonZero;nbeta];
    InonZero_complement = setdiff(1:nbeta,InonZero)';
    InonZero_diff = setxor(InonZero0,InonZero);
    numNonZero_diff = length(InonZero_diff);

    %----------------------------------------------------------------------
    % Calculate the second-order component
    %----------------------------------------------------------------------
    newtonSuccess=0;
    if (useNewton==1) && (length(InonZero)<=two_metric_threshold) 
        reducedGradient = Grad - lambda_globalv.*(beta_new<0) + ...
            lambda_globalv.*(beta_new>0);    % find the reduced Gradient
        reducedGradient(nbeta) = Grad(nbeta);
        reducedGradient = reducedGradient(InonZero);
                                           % shrink it to just the nonzeros
        partialHessian  = Hessian(beta, InonZero);
                                           % the partial Hessian
        scale_factor = mean(diag(partialHessian));      
        damping = min(scale_factor, 1.0*gpnorm);
        beta_step(InonZero) = -(partialHessian + ...
	    damping*eye(length(InonZero))) \ reducedGradient;
        beta_step(InonZero_complement) = beta_step_first(InonZero_complement);

        %------------------------------------------------------------------
        % Do a kind of line search
        %------------------------------------------------------------------
        newtonStepLength=1; 
        beta_newton=zeros(nbeta,1);
        beta_newton(InonZero) = beta(InonZero) + ...
            newtonStepLength*beta_step(InonZero);
                            % take the line-search Newton step
                            % in nonzero components;
        beta_newton(InonZero_complement)=0;
                            % set to zero those components that appear to 
                            % be zero in the first-order subproblem
        iCrossings=(beta_newton.*beta<0);
        iCrossings(nbeta)=0;
        numCrossings=sum(iCrossings);
                            % number of kinks that are crossed by this step
      
        %------------------------------------------------------------------
        % Evaluate function and check for decrease over the step obtained
        % in the first-morder model
        %------------------------------------------------------------------
        Tlambda_new = Functionsmt(beta_newton);
        if Tlambda_new< min(Tlambda_first, Tlambda)
            newtonSuccess=1;
        end
        
        %------------------------------------------------------------------
        % If kinks were crossed, rescale the step to stop at the first kink
        % evaluate function and check for decrease over the step from the
        % first-order model
        %------------------------------------------------------------------
        if numCrossings>0 && newtonSuccess==0
            findCrossings=find(iCrossings==1);
            newtonStepLength = min( abs(beta(findCrossings)) ./ ...
                (abs(beta(findCrossings)-beta_newton(findCrossings))) );
            beta_newton(InonZero) = ...
                beta(InonZero)+newtonStepLength*beta_step(InonZero);
            beta_newton(InonZero_complement)=0;
            Tlambda_new = Functionsmt(beta_newton);
            if Tlambda_new< min(Tlambda_first, Tlambda)
                newtonSuccess=1;
            end
        end

        %------------------------------------------------------------------
        % End of Newton segment. If a good step was found in this segment,
        % take it. Otherwise try some alternatives.
        %------------------------------------------------------------------
      
        if newtonSuccess
            beta_new = beta_newton;
            if Tlambda_first < Tlambda
                alpha = 0.8*alpha;       % did the first-order step give a 
                                         % decrease too? If so, decrease
                                         % alpha; otherwise increase it.
            else
                alpha = 2*alpha;
                if alpha > alphaMax
                    loop_on = 0;
                end
            end
        else                             % try just the first order step
            if Tlambda_first < Tlambda
                beta_step = beta_step_first;
                beta_new = beta + beta_step;
                alpha = 0.8*alpha;
            else
                beta_new = beta;
                alpha = 2*alpha;
                if alpha > alphaMax
                    loop_on = 0;
                end
            end
        end % newtonSuccess
    else                                 % useNewton = 0 or two metric 
                                         % threshold exceeded, 
                                         % just try the first-order step
        beta_step = beta_step_first;
        if Tlambda_first < Tlambda
            beta_new = beta + beta_step;
            alpha = 0.8*alpha;
        else
            beta_new = beta;
            alpha = 2*alpha;
            if alpha > alphaMax
                loop_on = 0;
            end
        end
    end
    Tlambda_new = Functionsmt(beta_new);   % reevaluate function
  
    %----------------------------------------------------------------------
    % Update everything for the next iteration
    %----------------------------------------------------------------------
    Grad_old = Grad;
    beta_step = beta_new - beta;
    beta = beta_new; 
    Tlambda = Tlambda_new; 
end % end the algorithm

[Tlambda, loglike] = Functionsmt(beta);   % reevaluate final function

numNonzeros_main = length(find(abs(beta(1:nmain(1)))>1e-6));
numNonzeros_inter = length(find(abs(beta)>1e-6))-1-numNonzeros_main;
if length(nmain) == 2
    numNonzeros_inter2 = length(find(abs(beta(1:(sum(nmain))))>1e-6));
    numNonzeros_inter3 = numNonzeros_inter - numNonzeros_inter2;
end


%--------------------------------------------------------------------------
% Calculate GACV or BGACV score
%--------------------------------------------------------------------------
logit=fBasis*beta;
prob=exp(logit)./(1+exp(logit));
OBS=-1/sampsize*sum(Y_global.*log(prob)+(1-Y_global).*log(1-prob));
W=diag(prob./(1+exp(logit)));
nzindex=find(abs(beta)>1e-6);
numNonzeros = length(nzindex);  
iterations = iter;
U = transpose(fBasis(:,nzindex))*W*fBasis(:,nzindex);
[EV EU] = eig(U);                       % use generalized inverse
eu = diag(EU);
smeig_ind = find(abs(eu)<1e-10);
inv_eu(smeig_ind) = 0;
inv_eu(setdiff(1:length(eu),smeig_ind)) = 1./eu(setdiff(1:length(eu)...
    ,smeig_ind));
H=fBasis(:,nzindex)*(EV*diag(inv_eu)*EV')*transpose(fBasis(:,nzindex));
if method == 2                          % GACV
    ff=OBS+1/sampsize*sum(Y_global.*(Y_global-prob))*trace(H)/...
        (sampsize-length(nzindex));
elseif method == 1                      % BGACV
    ff=OBS+log(sampsize)/sampsize/2*sum(Y_global.*(Y_global-prob))*...
        trace(H)/(sampsize-length(nzindex));
    if length(nmain) == 1
    ff = ff*(1+0.5*abs(numNonzeros_main-numNonzeros_inter)/...
        (numNonzeros_main+numNonzeros_inter));
    else
        numNonzeros_ave = (numNonzeros_main+numNonzeros_inter)/3;
    ff = ff*(1+0.5*(abs(numNonzeros_main-numNonzeros_ave) + ...
        abs(numNonzeros_inter2-numNonzeros_ave) + ...
        abs(numNonzeros_inter3-numNonzeros_ave))/...
        (numNonzeros_main+numNonzeros_inter));
    end
end
return
