function fm = glps(X,Y,nvargroup)
%Solves the bucket problem. For large dimentional X, divide the pairwise
%interactions of X into small buckets
global fBasis;
global Y_global;
method = 1; loglambda0 = -1.1; output = 0;
Y_global = Y;
[sampsize dim] = size(X);
if nargin < 3
    nvargroup = 200;
end
ngroup1 = floor(dim/nvargroup);
inx.main = [];
inx.inter = [];

t1 = clock;

%--------------------------------------------------------------------------
% On all squares
%--------------------------------------------------------------------------
for i = 1:ngroup1-1
    for j = i+1:ngroup1
        [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,j,1);
        fprintf(' variable 1 =%3d, variable 2 =%3d\n',i,j);
        beta = betaopt(method, loglambda0, output);
        inxcurr = getinx(beta,inxfullcurr);
        inx.main = [inx.main; inxcurr.main];
        inx.inter = [inx.inter; inxcurr.inter];
    end
end

%--------------------------------------------------------------------------
% On major triangles
%--------------------------------------------------------------------------
for i = (2:2:ngroup1) - 1
    [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,i+1,2);
    beta = betaopt(method, loglambda0, output);
    inxcurr = getinx(beta,inxfullcurr);
    inx.main = [inx.main; inxcurr.main];
    inx.inter = [inx.inter; inxcurr.inter];
end

%--------------------------------------------------------------------------
% On remaining rectangles
%--------------------------------------------------------------------------
for i = 1:ngroup1
    j = ngroup1 + 1;
    [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,j,3);
    beta = betaopt(method, loglambda0, output);
    inxcurr = getinx(beta,inxfullcurr);
    inx.main = [inx.main; inxcurr.main];
    inx.inter = [inx.inter; inxcurr.inter];
end

%--------------------------------------------------------------------------
% On remaining triangles
%--------------------------------------------------------------------------
if mod(ngroup1,2) == 1      % big&small
    i = ngroup1;
    j = ngroup1 + 1;
    [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,j,4);
else                        % small
    i = ngroup1 + 1;
    j = ngroup1 + 2;
    [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,j,5);
end
beta = betaopt(method, loglambda0, output);
inxcurr = getinx(beta,inxfullcurr);
inx.main = [inx.main; inxcurr.main];
inx.inter = [inx.inter; inxcurr.inter];
 

%--------------------------------------------------------------------------
% A whole model on selected variables
%--------------------------------------------------------------------------
if size(inx.inter,1) > 1
    inxg = unique([unique(inx.main); unique(inx.inter)]);
else
    inxg = unique([unique(inx.main)' unique(inx.inter)])';
                            % indices of variables selected by group step
end
save tmp_inxg.mat inxg
fmlps = lpsmt(X(:,inxg),Y);
inx.main = unique(inx.main);
fm.inxg = inx;


% return fm
model.coeff = fmlps.model.coeff;
model.se = fmlps.model.se;
model.coeffcorr = fmlps.model.coeffcorr;
model.p = fmlps.model.p;
model.prob = fmlps.model.prob;
fm.model = model;
pattern = fmlps.pattern;
pattern(pattern>0) = inxg(pattern(pattern>0));
fm.pattern = pattern;
t2 = clock;
info.runtime = etime(t2,t1);
fm.info = info;
return;

%--------------------------------------------------------------------------
% Get the index of main effects and interactions, after calculating beta
%--------------------------------------------------------------------------
function inx = getinx(beta,inxfullcurr)
global fBasis Y_global
nmainfullcurr = length(inxfullcurr.main);
inx.main = [];
inx.inter = [];
inxlasso = find(abs(beta)>1e-6)';
nsigblasso = length(inxlasso);
if nsigblasso > 1
    inxlasso = inxlasso(1:nsigblasso-1);
    XX = fBasis(:,inxlasso);
    [ncoe nstat inxlogist] = bgacvp(XX,Y_global);
    inxlps = inxlasso(inxlogist);
else
    inxlps = [];
end
ninxlps = length(inxlps);
if ninxlps > 0
    inx.main = inxfullcurr.main(intersect(1:nmainfullcurr,inxlps));
    intertmp = setdiff(inxlps,1:nmainfullcurr) - nmainfullcurr;
    inx.inter = inxfullcurr.inter(intertmp,:);
end
return
    
